--[[--------------------------------------------------------------------------------------------------
GameDataHandlerHandler

Version: 		003
Author: 		Torben Ratzlaff
Website: 		screenheadmedia.com

Purpose:
Administrates all game relevant data.
Saves and loads chosen datasets. (e.g. savegames, highscores etc.)

----------------------------------------------------------------------------------------------------]]


--[[--------------------------------------------------------------------------------------------------
How to use:
The GameDataHandler has a data table (GameDataHandler.data) in which you can store values you want to get access to it from another file.
You can save specific fields of this table to store them for the next playsession. All values can be soted except functions and userdata.
Previously saved values may be loaded in the next playsession.


--sets the gamedata to it's given default values (determined via GameDataHandler:setDefaultValues())
GameDataHandler:setDefaults()


--loads a specific previously saved table and returns it. Also references this value at GameDataHandler.name
GameDataHandler:load(name)
--name 			- name of the data table that should be loaded and referenced
--return 		- returns the loaded data


--saves data that has been stored in the GameDataHandler via GameDataHandler:setValue(...) or by directly accessing the GameDataHandler values
GameDataHandler:save(name)
--name 			- name of the data table that should be saved


--saves all values which names are set via GameDataHandler:setSaveValues(values)
GameDataHandler:saveAll()


--loads all values which names are set via GameDataHandler:setSaveValues(values)
GameDataHandler:loadAll()


--resets the stored data to it's defaults. Data defined by GameDataHandler:setResetExcludeValues(values) will be loaded afterwards. At the end it saves. (use this to reset savegames but keep options)
GameDataHandler:reset()


--defines the default values, that are set when you call GameDataHandler:setDefaults()
GameDataHandler:setDefaultValues({
	{"partialpath1", "partialpath2", value}		--sets the value at the given path. (e.g. GameDataHandler.partialpath1.partialpath2 = value) Creates table if necessary
})


--defines the data that is saved and loaded by GameDataHandler:saveAll() and GameDataHandler:loadAll()
GameDataHandler:setSaveValues(values)
--values 		-a table of strings. (e.g. "savegame" -> GameDataHandler.data.savegame will be saved and loaded)


--defines the data that will be restored after GameDataHandler:reset() is called.
GameDataHandler:setResetExcludeValues(values)
--values 		-a table of strings. (e.g. "savegame" -> GameDataHandler.savegame will be restored after reset)


--sets a value at a given path in GameDataHandler.data. You can also set the value directly.
GameDataHandler:setValue(...)
--... 			-e.g. "partialpath1", "partialpath2", value -> sets the value at the given path. (e.g. GameDataHandler.partialpath1.partialpath2 = value) Creates table if necessary


--gets a value from a given path in GameDataHandler.data. You can also get the value directly.
GameDataHandler:setValue(...) --gets the value from a given path
--... 			-e.g. "partialpath1", "partialpath2", value -> gets the value from a given path. (e.g. GameDataHandler.partialpath1.partialpath2)
--return 		-returns the value

----------------------------------------------------------------------------------------------------]]


local json = require("json")

local json_encode = json.encode
local json_decode = json.decode
local system_pathForFile = system.pathForFile
local system_DocumentsDirectory = system.DocumentsDirectory
local io_open = io.open
local io_close = io.close

local function saveTable(t, filename)
    local path = system_pathForFile( filename, system_DocumentsDirectory)
    local file = io_open(path, "w")

    if file then
        local contents = json_encode(t)
        file:write( contents )
        io_close( file )
        return true
    end

    return false
end

local function loadTable(filename)
    local path = system_pathForFile( filename, system_DocumentsDirectory)
    local contents = ""
    local myTable = {}
    local file = io_open( path, "r" )

    if file then
         local contents = file:read( "*a" )
         myTable = json_decode(contents);
         io_close( file )
         return myTable 
    end
end



local GameDataHandler = {}
local saveData = {}
GameDataHandler.data = saveData

local saveValues = {}
local defaultValues = {}
local resetExcludeValues = {}


function GameDataHandler:setDefaults()	
	for i=1, #defaultValues do
		local defaultValue = defaultValues[i]
		local defaultValueNum = #defaultValue

		local slot = saveData

		for i=1, defaultValueNum-2 do
			local slotName = defaultValue[i]

			if i < defaultValueNum-1 then
				if not slot[slotName] then
					slot[slotName] = {}
				end
			end

			slot = slot[slotName]
		end

		local slotValue = defaultValue[defaultValueNum]
		if slotValue == "nil" or defaultValueNum <= 1 then
			slotValue = nil
		end

		slot[defaultValue[defaultValueNum-1]] = slotValue
	end
end


function GameDataHandler:load(name)
	local loadedTable = loadTable(name)
	
	if loadedTable then
		saveData[name] = loadedTable
	end

	return loadedTable
end

function GameDataHandler:save(name)
	saveTable(saveData[name], name)
end

function GameDataHandler:saveAll()
	for i=1, #saveValues do
		self:save(saveValues[i])
	end
end

function GameDataHandler:loadAll()
	for i=1, #saveValues do
		self:load(saveValues[i])
	end
end

function GameDataHandler:reset()
	self:setStandards()
	
	for i=1, #resetExcludeValues do
		self:load(resetExcludeValues[i])
	end
	
	self:saveAll()
end

function GameDataHandler:setDefaultValues(values)
	defaultValues = values
end

function GameDataHandler:setSaveValues(values)
	saveValues = values
end

function GameDataHandler:setResetExcludeValues(values)
	resetExcludeValues = values
end

function GameDataHandler:setValue(...)
	local argNum = #arg
	local slot = saveData

	for i=1, argNum-2 do
		local slotName = arg[i]

		if i < argNum-1 then
			if not slot[slotName] then
				slot[slotName] = {}
			end
		end

		slot = slot[slotName]
	end

	local slotValue = arg[argNum]
	if slotValue == "nil" or argNum == 1 then
		slotValue = nil
	end

	slot[arg[argNum-1]] = slotValue
end

function GameDataHandler:getValue(...)
	local argNum = #arg
	local slot = saveData

	for i=1, argNum do
		slot = slot[arg[i]]

		if not slot then
			return nil
		end
	end

	return slot
end

return GameDataHandler
